define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/event",
    "dojo/NodeList-traverse",
    "dojo/dom-attr",
    "dojo/dom-style",
    "dojo/dom-class",
    "dojo/dom-geometry",
    "dojo/query",
    "dojo/text!./templates/GlobalMenu.html",
    "dojo/when",
    "dojo/dom-construct",
    "dijit/focus",
    "dijit/_Container",
    "dijit/_TemplatedMixin",
    "dijit/_Widget",
    "dijit/registry",
    "epi/dependency",
    "epi/shell/_ContextMixin",
    "epi/Url",
    "epi/i18n!epi/shell/ui/nls/EPiServer.Shell.UI.Resources.GlobalMenu",
    "dojo/has",
    "dgrid/util/has-css3"],

function (
    declare,
    lang,
    event,
    traverse,
    domAttr,
    domStyle,
    domClass,
    domGeometry,
    query,
    template,
    when,
    domConstruct,
    focusManager,
    _Container,
    _TemplatedMixin,
    _Widget,
    registry,
    dependency,
    _ContextMixin,
    Url,
    resources,
    has) {

    return declare([_Widget, _TemplatedMixin, _Container, _ContextMixin], {
        // tags:
        //    internal

        res: resources,
        templateString: template,

        _focusNode: null,
        _siteUrl: null,
        _profile: null,
        _epiPinStateKey: "epi-menupinstate",

        openClass: "epi-globalNavigation--open",
        hoverClass: "epi-globalNavigation--hover",
        shadowClass: "epi-globalNavigation--shadow",
        hiddenClass: "dijitHidden",
        collapseButtonClass: "epi-navigation-expandcollapseBtn",
        collapseButtonOpenClass: "epi-navigation-expandcollapseBtn--open",
        collapseButtonPinnedClass: "epi-navigation-expandcollapseBtn",
        collapseIconClass: "epi-navigation-expandcollapseIcon",
        collapseIconSelectedOpenClass: "epi-navigation-expandcollapseIcon--open",
        collapseIconPinnedClass: "epi-navigation-expandcollapseIcon--pinned",

        postCreate: function () {
            // summary:
            //      Bind global menu events.
            // tags:
            //      public

            this.inherited(arguments);

            // Get hold of the profile
            this._profile = dependency.resolve("epi.shell.Profile");

            // The node we're focusing when the menu is shown
            this._focusNode = query(".epi-navigation-selected > a", this.containerNode)[0];

            // Hide as soon as this widget looses focus
            this.connect(this, "onBlur", function () {
                this._toggleGlobalSearch(false);
                this._hideMenu();
            });

            // Show more of global navigation button when user hovers
            this.connect(this.domNode, "onmouseenter", this._toggleHintOn);
            this.connect(this.domNode, "onmouseleave", this._toggleHintOff);
            // Handle toggle when clicking on expand-collapse icon.
            this.connect(this.accordionNode, "onclick", this._showMenu);
            this.connect(this.expandCollapseContainer, "onclick", this._toggleMenu);

            when(this.getCurrentContext()).then(function (currentContext) {
                this.contextChanged(currentContext);
            }.bind(this));

            //Check if the browser supports css transitions before using it
            if (has("css-transitions")) {
                this.connect(this.accordionNode, has("transitionend"), lang.hitch(this, function () {
                    if (this._isOpen()) {
                        focusManager.focus(this._focusNode);
                        return;
                    }

                    if (domClass.contains(this.accordionNode, this.hoverClass)) {
                        return;
                    }

                    domClass.add(this.containerNode, this.hiddenClass);
                }));
            }
        },

        startup: function () {
            // summary:
            //      Used to work out and set the pinned state of the menu
            // tags:
            //      public

            this.inherited(arguments);

            // REMARKS: I dont like that we have a "dependency" to one of it's siblings in the DOM,
            // But I don't have a better idea right now that won't need Find to update their code. Will think more about it
            this._rootContainer = registry.byId("rootContainer");

            this._isMenuPinned().then(function (pinned) {
                // Hide the menu if its not pinned
                if (pinned) {
                    this._pinMenu();
                } else {
                    this._hideMenu();
                }
            }.bind(this));
        },

        _getContentHeight: function () {
            // summary:
            //      Returns the height of the contained content

            return domGeometry.getContentBox(this.containerNode).h;
        },

        _toggleMenu: function (e) {
            // summary:
            //      Toggle display state of global menu.
            // tags:
            //      private

            if (this._isOpen()) {
                this._isMenuPinned().then(function (pinned) {
                    if (pinned) {
                        // Make button an arrow again
                        this._unpinMenu();
                        this._hideMenu();
                    } else {
                        this._pinMenu();
                    }
                }.bind(this));
            } else {
                this._showMenu();
            }
            event.stop(e);
        },

        _toggleHintOn: function () {

            if (this._isOpen()) {
                return;
            }
            domClass.add(this.accordionNode, this.hoverClass);
        },

        _toggleHintOff: function () {
            if (this._isOpen()) {
                return;
            }
            domClass.remove(this.accordionNode, this.hoverClass);
        },

        _showMenu: function () {
            // summary:
            //      Show global menu.
            // tags:
            //      private

            if (this._isOpen()) {
                return;
            }

            domClass.remove(this.accordionNode, this.hoverClass);
            domClass.add(this.accordionNode, this.openClass);

            domClass.remove(this.containerNode, this.hiddenClass);
            domStyle.set(this.accordionNode, "height", this._getContentHeight() + "px");
            domClass.add(this.domNode, this.shadowClass);

            // Show the "menu can be pinned" icon
            domClass.replace(
                this.expandCollapseButton.firstChild,
                this.collapseIconSelectedOpenClass,
                [this.collapseIconPinnedClass, this.collapseIconClass]);

            // Make the button bigger so its easier to click
            domClass.replace(
                this.expandCollapseButton,
                this.collapseButtonOpenClass,
                [this.collapseButtonClass, this.collapseButtonPinnedClass]);
        },

        _hideMenu: function () {
            // summary:
            //      Hide global menu.
            // tags:
            //      private

            this._isMenuPinned().then(function (pinned) {
                if (pinned) {
                    return;
                }
                domClass.remove(this.accordionNode, this.hoverClass);
                domClass.remove(this.accordionNode, this.openClass);

                domClass.remove(this.domNode, this.shadowClass);
                domStyle.set(this.accordionNode, "height", "0px");

                // Show the default open menu icon
                domClass.replace(
                    this.expandCollapseButton.firstChild,
                    this.collapseIconClass,
                    [this.collapseIconSelectedOpenClass, this.collapseIconPinnedClass]);

                // Make the button small
                domClass.replace(
                    this.expandCollapseButton,
                    this.collapseButtonClass,
                    [this.collapseButtonOpenClass, this.collapseButtonPinnedClass]);
            }.bind(this));
        },

        _isOpen: function () {
            return domClass.contains(this.accordionNode, this.openClass);
        },

        _getToViewModeElement: function () {
            // summary:
            //      Get to view mode element
            //
            // tags:
            //      private

            var utilItemsContainer = query(".epi-navigation-container-utils", this.element)[0];
            return query(".epi-navigation-global_sites.epi-navigation-currentSite.epi-navigation-iconic", utilItemsContainer)[0];
        },

        contextChanged: function (ctx, callerData) {
            // summary:
            //      Update direct URL (navigator current sites for instance)
            // ctx: Object
            //      Current page context
            // tags:
            //      internal

            var toViewModeElement = this._getToViewModeElement();
            //Only change the URL if we have one site
            if (!toViewModeElement) {
                return;
            }

            var url = this._buildPublicUrl(ctx, toViewModeElement);
            var title = lang.replace(this.res.toviewmode, [url.toString()]);

            domAttr.set(toViewModeElement, { href: url.toString(), title: title });
        },

        _buildPublicUrl: function (ctx, viewModeElement) {
            // summary:
            //      Build the public URL by current context

            //Copy the site URL form the element before we replace it
            if (this._siteUrl === null) {
                this._siteUrl = viewModeElement.href;
            }

            return new Url(ctx.publicUrl || this._siteUrl);
        },

        _toggleGlobalSearch: function (/* Boolean */visible) {
            // summary:
            //      Show or hide the current global search box
            // remarks:
            //      Currently global search is written using jQuery, so we have to access search box by ID

            var searchBox = query("#epi-searchContainer")[0];
            if (searchBox) {
                domStyle.set(searchBox, "display", visible ? "block" : "none");
            }
        },

        _pinMenu: function () {
            // summary:
            //      Pin the menu open
            // remarks:
            //      Can be used on init and/or during user interaction
            // tags:
            //      private

            // Toggle the cookie to be set or unset
            this._toggleMenuState(true);

            // Open the menu
            this._showMenu();

            // Make sure the menu doesn't float
            domStyle.set(this.domNode, "position", "relative");

            // Remove the shadow effect
            domClass.remove(this.domNode, this.shadowClass);

            // Show the "is pinned" icon
            domClass.replace(
                this.expandCollapseButton.firstChild,
                this.collapseIconPinnedClass,
                this.collapseIconClass);

            // Make the button small again
            domClass.replace(
                this.expandCollapseButton,
                this.collapseButtonPinnedClass,
                [this.collapseButtonClass, this.collapseButtonOpenClass]);

            // Make sure the main content container is 100% high minus the height of the menu
            var newHeight = "calc(100% - " + this._getContentHeight() + "px)";
            domStyle.set(this._rootContainer.domNode, "height", newHeight);
            this._rootContainer.resize();
        },

        _unpinMenu: function () {
            // summary:
            //      Unpin the menu
            // remarks:
            //      Can be used on init and/or during user interaction
            // tags:
            //      private

            // Make sure the main content container is a height of 100%
            domStyle.set(this._rootContainer.domNode, "height", "100%");
            this._rootContainer.resize();

            // Remove the shadow effect
            domClass.add(this.domNode, this.shadowClass);

            // Toggle the state to be set or unset
            this._toggleMenuState(false);

            // Make the menu float again
            domStyle.set(this.domNode, "position", "absolute");

            // Hide the menu
            this._hideMenu();

            // Show the hint button
            domStyle.set(this.expandCollapseButton, "display", "block");

            // Show the unpinned icon
            domClass.replace(
                this.expandCollapseButton.firstChild,
                this.collapseIconClass,
                this.collapseIconPinnedClass);
        },

        _toggleMenuState: function (pinned) {
            // summary:
            //      Toggle the menu state as open or closed
            // remarks:
            //      Uses the Episerver profile to persist the toggle state
            // tags:
            //      private

            this._profile.set(this._epiPinStateKey, pinned);
        },

        _isMenuPinned: function () {
            // summary:
            //      Check if the menu is pinned out
            // remarks:
            //      Uses the Episerver profile to check if the menu is pinned or not
            // tags:
            //      private

            return when(this._profile.get(this._epiPinStateKey)).then(function (pinned) {
                if (pinned && pinned === true) {
                    return pinned;
                }
                return false;
            });
        }
    });
});
